/*
 * Copyright (c) 2009 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef _RMP400STRUCT_H
#define _RMP400STRUCT_H

typedef enum RMP400_MODE {
	RMP400_IDLE = 0,
	RMP400_MOTORS_OFF,
	RMP400_MANUAL,
	RMP400_TRACK,
	RMP400_EMERGENCY
} RMP400_MODE;

typedef struct RMP400_DATA_STR {
	double pitch_angle;
	double pitch_rate;
	double roll_angle;
	double roll_rate;
	double lw_velocity;
	double rw_velocity;
	double yaw_rate;
	short servo_frames;
	double integrated_left_wheel;
	double integrated_right_wheel;
	double integrated_fore_aft;
	double integrated_yaw;
	double left_torque;
	double right_torque;
	short operational_mode;
	short controller_gain_schedule;
	double ui_voltage;
	double powerbase_voltage;
	double battery_charge;
	double velocity_command;
	double turn_command;
} RMP400_DATA_STR;
       
typedef struct RMP400_STATUS_STR {
	RMP400_MODE rs_mode;
	RMP400_DATA_STR rs_data[2];
} RMP400_STATUS_STR;

/* Kinematics of the robot */
typedef struct RMP400_KINEMATICS_STR {
	double leftWheelRadius;	
	double rightWheelRadius;
	double axisWidth;
} RMP400_KINEMATICS_STR;

/* Coefficient proportionnel a la vitesse pour calculer l'erreur sur 
   le deplacement des roues */
typedef struct RMP400_VAR_PARAMS {
	double coeffLin;
	double coeffAng;
	double coeffLinAng;
} RMP400_VAR_PARAMS;

/* Track parameters */
typedef struct RMP400_TRACK_PARAMS {
	GENPOS_POSTER_NAME posterName;
	GENPOS_TRACK_MODE mode;
} RMP400_TRACK_PARAMS;

/*----------------------------------------------------------------------*/
/* 3D */

/* a GENPOS_CART_STATE-like struct in 3D */
/* Same thing with 4 more elements: z,zC,pitch,roll */
typedef struct RMP400_CART_3DSTATE {
	double xRef;                  /* position  */
	double yRef;                  /* control point */
	double zRef;
	double xRob;                 /* position */
	double yRob;                 /* robot center */
	double zRob;
	double theta;              /* robot orientation*/
	double pitch;              /* robot's pitch (site) */
	double roll;               /* robot's roll (gite) */
	double v;                  /* linear velocity */
	double vt;
	double w;                  /* angular velocity */
} RMP400_CART_3DSTATE;

typedef struct RMP400_CART_3DCONFIG {
	double x;
	double y;
	double z;
	double theta;
} RMP400_CART_3DCONFIG;

/*----------------------------------------------------------------------*/
/* Odometry Method */

typedef enum RMP400_ODOMETRY_MODE {
	RMP400_ODO_2D, /* Standard 2D odometry */
	RMP400_ODO_3D  /* 3D odometry using Attitude Estimation 
			  provided by IMU */
} RMP400_ODOMETRY_MODE;

#define	RMP400_MAX_STRLEN	64
#define RMP400_DEFAULT_ATTITUDE_POSTER "imuAttitudeEstimation"

typedef struct RMP400_ODOMETRY_METHOD {
	RMP400_ODOMETRY_MODE mode;
	int unused;
	char attitudePosterName[RMP400_MAX_STRLEN];
} RMP400_ODOMETRY_METHOD;

/*----------------------------------------------------------------------*/
/* Gyro */

typedef enum RMP400_GYRO_MODE {
	RMP400_GYRO_OFF,
	RMP400_GYRO_ON_IF_MOTION,
	RMP400_GYRO_ON
} RMP400_GYRO_MODE;

typedef struct RMP400_GYRO_PARAMS {
	RMP400_GYRO_MODE mode;
	char port[32];
} RMP400_GYRO_PARAMS;

typedef struct RMP400_GYRO {
	RMP400_GYRO_PARAMS params;
	int initDone;
	int gyroOn;
	RMP400_GYRO_MODE currentMode;
	double gyroToRobotOffset;
	double gyroTheta;
	double robotTheta;
	double gyroOmega;
#if (GYRO==gyroKVH1000Task)
	double gyroTemperature;
#endif
} RMP400_GYRO;


typedef struct RMP400_GYRO_ASSERV {
	int enabled; // is asserv enabled
	
	int first; // first call to this function ?
	int straight; // is in straight mode ?
	double straight_angle; // angle for straight mode
	double prev_reference;
	double prev_measure;
	double prev_command;
	double prev_t;
	double integral_s;
} RMP400_GYRO_ASSERV;

typedef struct RMP400_MAX_ACCEL {
	double prev_vel_command, prev_ang_command;
	double prev_vel_command_t, prev_ang_command_t;
} RMP400_MAX_ACCEL;

#endif
